﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Windows;
using System.ServiceModel;
using System.ServiceModel.Channels;
using Contracts;
using System.Threading;
using System.ServiceModel.Description;
using System.Workflow.Runtime;
using System.Workflow.Runtime.Hosting;

namespace HostApplication
{
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App : Application
    {
        internal LocalWorkflowServiceHost<WorkflowsAndActivities.OrderWorkflow, HostService> localWorkflowHost;
        
        WorkflowServiceHost workflowHost;
        internal ServiceHost appHost;

        internal System.ServiceModel.Channels.Binding LocalBinding = new NetNamedPipeContextBinding(NetNamedPipeSecurityMode.None);
        internal string WFAddress = "net.pipe://localhost/OrderWF";
        internal string HostAddress = "net.pipe://localhost/WPFHostApp";
        protected override void OnStartup(StartupEventArgs e)
        {
            base.OnStartup(e);
            ConfigureHosts();
        }

        protected override void OnExit(ExitEventArgs e)
        {
            //combined custom host to encapsulate details
            if (localWorkflowHost != null)
                if (localWorkflowHost.State == CommunicationState.Opened)
                    localWorkflowHost.Close();
                else
                    localWorkflowHost.Abort();

            //hosts for simple examples (window1.xaml and window2.xaml)
            if (workflowHost != null)
                if (workflowHost.State == CommunicationState.Opened)
                    workflowHost.Close();
                else
                    workflowHost.Abort();

            if (appHost != null)
                if (appHost.State == CommunicationState.Opened)
                    appHost.Close();
                else
                    appHost.Abort();

            base.OnExit(e);
        }


        private void ConfigureHosts()
        {
            ThreadPool.QueueUserWorkItem((o) =>
            {
                localWorkflowHost = new LocalWorkflowServiceHost<WorkflowsAndActivities.OrderWorkflow,
                    HostService>();
                localWorkflowHost.Description.Behaviors.Find<ServiceDebugBehavior>().IncludeExceptionDetailInFaults = true;

                localWorkflowHost.Open();
                
                //host the workflow
                workflowHost = new WorkflowServiceHost(typeof(WorkflowsAndActivities.OrderWorkflow));
                workflowHost.AddServiceEndpoint(
                    "Contracts.IWorkflowInterface",
                    LocalBinding, WFAddress);
                //add the client endpoint for the host service
                ServiceEndpoint endpoint = new ServiceEndpoint
                (
                    ContractDescription.GetContract(typeof(Contracts.IHostInterface)),
                    LocalBinding, new EndpointAddress(HostAddress)
                );
                endpoint.Name = "HostEndpoint-Contracts.IHostInterface";

                //by using the channel manager, no app.config is required
                WorkflowRuntime runtime = workflowHost.Description.Behaviors.Find<WorkflowRuntimeBehavior>().WorkflowRuntime;
                ChannelManagerService chanMan =
                    new ChannelManagerService(new
                    List<ServiceEndpoint>
                    {
                        endpoint
                    });

                runtime.AddService(chanMan);

                try
                {
                    workflowHost.Open();
                }
                catch (Exception ex)
                {
                    workflowHost.Abort();
                    MessageBox.Show(String.Format("There was an error hosting the workflow as a service: {0}", ex.Message));
                }
                
            }   
            );

            //host the local service
            appHost = new ServiceHost(new HostService());
            appHost.AddServiceEndpoint("Contracts.IHostInterface",
                LocalBinding, HostAddress);

            try
            {
                appHost.Open();
            }
            catch (Exception ex)
            {
                appHost.Abort();
                MessageBox.Show(String.Format("There was an error hosting the local service: {0}", ex.Message));
            }
        }

        
    }
}
