﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Contracts;
using System.ServiceModel;
using System.Threading;
using System.ServiceModel.Description;
using System.Workflow.Runtime;
using System.Workflow.Runtime.Hosting;
using System.ServiceModel.Channels;

namespace HostApplication
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window2 : Page
    {
        private IWorkflowInterface updateProxy;
        private ChannelFactory<IWorkflowInterface> factory;

        public Window2()
        {
            InitializeComponent();
        }

       
        private void Button_Click(object sender, RoutedEventArgs e)
        {
            IWorkflowInterface proxy = GetProxy();

            proxy.SubmitOrder(
                new Order
                {
                    CustomerName = "Matt",
                    OrderID = 0,
                    OrderTotal = 250.00
                });

            GetWorkflowInstance(proxy);
            ((IClientChannel)proxy).Close();
        }

        

        private void GetWorkflowInstance(IWorkflowInterface proxy)
        {
            IContextManager mgr = ((IClientChannel)proxy).GetProperty<IContextManager>();
            
            string wfID = mgr.GetContext()["instanceId"];
            wfIdText.Text = wfID;
        }

        private IWorkflowInterface GetProxy()
        {
            App a = (App)Application.Current;
            if(factory == null)
                factory = new ChannelFactory<IWorkflowInterface>(a.LocalBinding, a.WFAddress);
            IWorkflowInterface proxy = factory.CreateChannel();
            ((IClientChannel)proxy).Open();
            return proxy;
        }

        private IWorkflowInterface GetProxyForUpdate()
        {
            App a = (App)Application.Current;

            if (updateProxy == null)
            {
                if (factory == null)
                    factory = new ChannelFactory<IWorkflowInterface>(a.LocalBinding, a.WFAddress);

                updateProxy = factory.CreateChannel();
                IContextManager mgr = ((IClientChannel)updateProxy).GetProperty<IContextManager>();
                mgr.Enabled = false;
                ((IClientChannel)updateProxy).Open();
            }

            return updateProxy;
        }

        private void Button_Click_1(object sender, RoutedEventArgs e)
        {

            IWorkflowInterface proxy = GetProxyForUpdate();
            using (OperationContextScope scope = new OperationContextScope((IContextChannel)proxy))
            {
                ContextMessageProperty property =
                    new ContextMessageProperty(
                        new Dictionary<string, string>
                        {
                            {"instanceId", wfIdText.Text}
                        });
                OperationContext.Current.OutgoingMessageProperties.Add("ContextMessageProperty", property);
                
                proxy.UpdateOrder(
                    new Order
                    {
                        CustomerName = "Matt",
                        OrderID = 2,
                        OrderTotal = 250.00,
                        OrderStatus = "Updated"
                    });
            }
        }

        private void SetupProxyForUpdate(string instanceID)
        {
            ContextMessageProperty property =
              new ContextMessageProperty(
                  new Dictionary<string, string>
                    {
                        {"instanceId", instanceID}
                    });
            OperationContext.Current.OutgoingMessageProperties.Add("ContextMessageProperty", property);

        }

        private void SetWorkflowIDOnProxy(IWorkflowInterface proxy)
        {
            IContextManager mgr = ((IClientChannel)proxy).GetProperty<IContextManager>();
            mgr.SetContext(new Dictionary<string, string>{
                {"instanceId", wfIdText.Text}
            });
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            HostService host =  ((App)Application.Current).appHost.SingletonInstance as HostService;
            if (host != null)
                this.DataContext = host;
        }
    }
}
